@@ -439,9 +439,8 @@ gconf_client_get_default (void)
   else
     {
       client = g_object_new (gconf_client_get_type (), NULL);
-
+      g_object_ref (G_OBJECT (client));      
       set_engine (client, engine);      
-
       register_client (client);
     }
   
@@ -811,22 +810,6 @@ recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
   g_slist_free(subdirs);
 }
 
-
-typedef struct
-{
-  gboolean found_parent;
-  const char *key;
-} CheckUnderneathData;
-
-static void
-foreach_check_underneath (gpointer key, gpointer value, gpointer user_data)
-{
-  CheckUnderneathData *cud = user_data;
-  
-  if (gconf_key_is_below (key, cud->key))
-    cud->found_parent = TRUE;
-}
-
 static gboolean
 key_being_monitored (GConfClient *client,
                      const char  *key)
@@ -1438,7 +1421,6 @@ gchar*
 gconf_client_get_string(GConfClient* client, const gchar* key,
                         GError** err)
 {
-  static const gchar* def = NULL;
   GError* error = NULL;
   GConfValue* val;
 
@@ -1453,16 +1435,9 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
       g_assert(error == NULL);
       
       if (check_type (key, val, GCONF_VALUE_STRING, &error))
-        /* we cheat here (look below) so we have to cast this */
-        retval = (gchar *)gconf_value_get_string (val);
+        retval = gconf_value_steal_string (val);
       else
         handle_error (client, error, err);
-
-      /* This is a cheat; don't copy */
-      if (retval != NULL)
-        val->d.string_data = NULL; /* don't delete the string we are returning */
-      else
-        retval = def ? g_strdup (def) : NULL;
       
       gconf_value_free (val);
 
@@ -1472,7 +1447,7 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
     {
       if (error != NULL)
         handle_error (client, error, err);
-      return def ? g_strdup (def) : NULL;
+      return NULL;
     }
 }
 
@@ -1530,13 +1505,9 @@ gconf_client_get_schema  (GConfClient* client,
       g_assert(error == NULL);
       
       if (check_type (key, val, GCONF_VALUE_SCHEMA, &error))
-        retval = gconf_value_get_schema (val);
+        retval = gconf_value_steal_schema (val);
       else
         handle_error (client, error, err);
-
-      /* This is a cheat; don't copy */
-      if (retval != NULL)
-        val->d.schema_data = NULL; /* don't delete the schema */
       
       gconf_value_free (val);
 
@@ -2356,6 +2327,9 @@ static void
 notify_one_entry (GConfClient *client,
                   GConfEntry  *entry)
 {
+  g_object_ref (G_OBJECT (client));
+  gconf_entry_ref (entry);
+  
   /* Emit the value_changed signal before notifying specific listeners;
    * I'm not sure there's a reason this matters though
    */
@@ -2376,6 +2350,9 @@ notify_one_entry (GConfClient *client,
                               notify_listeners_callback,
                               &cae);
     }
+
+  gconf_entry_unref (entry);
+  g_object_unref (G_OBJECT (client));
 }
 
 static void
