@@ -55,6 +55,7 @@ typedef struct _GConfValue GConfValue;
 
 struct _GConfValue {
   GConfValueType type;
+  /* this is all PRIVATE dangit */      
   union {
     gchar* string_data;
     gint int_data;
@@ -72,15 +73,15 @@ struct _GConfValue {
   } d;
 };
 
-#define gconf_value_get_string(x)    ((const gchar*)(((GConfValue*)(x))->d.string_data))
-#define gconf_value_get_int(x)       (((GConfValue*)(x))->d.int_data)
-#define gconf_value_get_float(x)     (((GConfValue*)(x))->d.float_data)
-#define gconf_value_get_list_type(x) (((GConfValue*)(x))->d.list_data.type)
-#define gconf_value_get_list(x)      (((GConfValue*)(x))->d.list_data.list)
-#define gconf_value_get_car(x)       (((GConfValue*)(x))->d.pair_data.car)
-#define gconf_value_get_cdr(x)       (((GConfValue*)(x))->d.pair_data.cdr)
-#define gconf_value_get_bool(x)      (((GConfValue*)(x))->d.bool_data)
-#define gconf_value_get_schema(x)    (((GConfValue*)(x))->d.schema_data)
+const char*    gconf_value_get_string    (const GConfValue *value);
+int            gconf_value_get_int       (const GConfValue *value);
+double         gconf_value_get_float     (const GConfValue *value);
+GConfValueType gconf_value_get_list_type (const GConfValue *value);
+GSList*        gconf_value_get_list      (const GConfValue *value);
+GConfValue*    gconf_value_get_car       (const GConfValue *value);
+GConfValue*    gconf_value_get_cdr       (const GConfValue *value);
+gboolean       gconf_value_get_bool      (const GConfValue *value);
+GConfSchema*   gconf_value_get_schema    (const GConfValue *value);
 
 GConfValue* gconf_value_new                  (GConfValueType type);
 
@@ -127,6 +128,10 @@ gchar*      gconf_value_to_string            (const GConfValue* value);
  * that may apply to this key.
  */
 
+/* FIXME GConfMetaInfo is basically deprecated in favor of stuffing this
+ * info into GConfEntry, though the transition isn't complete.
+ */
+
 typedef struct _GConfMetaInfo GConfMetaInfo;
 
 struct _GConfMetaInfo {
@@ -135,18 +140,19 @@ struct _GConfMetaInfo {
   GTime  mod_time; /* time of the modification */
 };
 
-#define gconf_meta_info_get_schema(x)    ((const gchar*)((GConfMetaInfo*)(x))->schema)
-#define gconf_meta_info_get_mod_user(x)  (((GConfMetaInfo*)(x))->mod_user)
-#define gconf_meta_info_mod_time(x)  (((GConfMetaInfo*)(x))->mod_time)
+const char* gconf_meta_info_get_schema   (GConfMetaInfo *gcmi);
+const char* gconf_meta_info_get_mod_user (GConfMetaInfo *gcmi);
+GTime       gconf_meta_info_mod_time     (GConfMetaInfo *gcmi);
+
+GConfMetaInfo* gconf_meta_info_new          (void);
+void           gconf_meta_info_free         (GConfMetaInfo *gcmi);
+void           gconf_meta_info_set_schema   (GConfMetaInfo *gcmi,
+                                             const gchar   *schema_name);
+void           gconf_meta_info_set_mod_user (GConfMetaInfo *gcmi,
+                                             const gchar   *mod_user);
+void           gconf_meta_info_set_mod_time (GConfMetaInfo *gcmi,
+                                             GTime          mod_time);
 
-GConfMetaInfo* gconf_meta_info_new         (void);
-void           gconf_meta_info_free     (GConfMetaInfo* gcmi);
-void           gconf_meta_info_set_schema  (GConfMetaInfo* gcmi,
-                                            const gchar* schema_name);
-void           gconf_meta_info_set_mod_user(GConfMetaInfo* gcmi,
-                                            const gchar* mod_user);
-void           gconf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
-                                            GTime mod_time);
 
 
 /* Key-value pairs; used to list the contents of
@@ -156,23 +162,26 @@ void           gconf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
 typedef struct _GConfEntry GConfEntry;
 
 struct _GConfEntry {
-  gchar* key;
-  GConfValue* value;
-  gchar* schema_name;
+  char *key;
+  GConfValue *value;
+  char *schema_name;
   int pad1;
   gpointer pad2;
+  gpointer pad3;
+  GTime pad4;
+  int refcount;
   guint is_default : 1;
   guint is_writable : 1;
-  guint pad3 : 1;
-  guint pad4 : 1;
   guint pad5 : 1;
+  guint pad6 : 1;
+  guint pad7 : 1;
 };
 
-#define     gconf_entry_get_key(x)         ((const gchar*)((GConfEntry*)(x))->key)
-#define     gconf_entry_get_value(x)       (((GConfEntry*)(x))->value)
-#define     gconf_entry_get_schema_name(x) ((const gchar*)((GConfEntry*)(x))->schema_name)
-#define     gconf_entry_get_is_default(x)  (((GConfEntry*)(x))->is_default)
-#define     gconf_entry_get_is_writable(x) (((GConfEntry*)(x))->is_writable)
+const char* gconf_entry_get_key         (const GConfEntry *entry);
+GConfValue* gconf_entry_get_value       (const GConfEntry *entry);
+const char* gconf_entry_get_schema_name (const GConfEntry *entry);
+gboolean    gconf_entry_get_is_default  (const GConfEntry *entry);
+gboolean    gconf_entry_get_is_writable (const GConfEntry *entry);
 
 GConfEntry* gconf_entry_new              (const gchar *key,
                                           const GConfValue  *val);
