@@ -25,9 +25,10 @@
 #include <popt.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
-#include <libxml/xmlmemory.h>
+#include <libxml/globals.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <locale.h>
 
 static int set_mode = FALSE;
 static int get_mode = FALSE;
@@ -383,7 +384,7 @@ static int do_long_docs (GConfEngine *conf, const gchar **args);
 static int do_get_schema_name (GConfEngine *conf, const gchar **args);
 static int do_associate_schema (GConfEngine *conf, const gchar **args);
 static int do_dissociate_schema (GConfEngine *conf, const gchar **args);
-static int do_get_default_source (GConfEngine *conf, const gchar **args);
+static int do_get_default_source (const gchar **args);
 
 int 
 main (int argc, char** argv)
@@ -395,6 +396,8 @@ main (int argc, char** argv)
 
   LIBXML_TEST_VERSION;
   xmlKeepBlanksDefault(1);
+
+  setlocale (LC_ALL, "");
   
   ctx = poptGetContext("gconftool", argc, (const char **) argv, options, 0);
 
@@ -575,7 +578,7 @@ main (int argc, char** argv)
   if (default_source_mode)
     {
       const gchar** args = poptGetArgs(ctx);
-      if (do_get_default_source (conf, args)  == 1)
+      if (do_get_default_source (args)  == 1)
         return 1;
       else
         return 0;
@@ -612,7 +615,7 @@ main (int argc, char** argv)
       /* shut down daemon, this is a race condition, but will usually work. */
       gconf_shutdown_daemon (NULL);
     }
-  
+
   if (config_source == NULL)
     conf = gconf_engine_get_default();
   else
@@ -1104,7 +1107,7 @@ read_value_type(const char *string)
       break;
     case 's':
     case 'S':
-      switch (value_type[1])
+      switch (string[1])
 	{
 	case 't':
 	case 'T':
@@ -1115,7 +1118,7 @@ read_value_type(const char *string)
 	  type = GCONF_VALUE_SCHEMA;
 	  break;
 	default:
-	  fprintf(stderr, _("Don't understand type `%s'\n"), value_type);
+	  fprintf(stderr, _("Don't understand type `%s'\n"), string);
 	}
       break;
     case 'l':
@@ -1127,7 +1130,7 @@ read_value_type(const char *string)
       type = GCONF_VALUE_PAIR;
       break;
     default:
-      fprintf(stderr, _("Don't understand type `%s'\n"), value_type);
+      fprintf(stderr, _("Don't understand type `%s'\n"), string);
     }
   return type;
 }
@@ -2637,7 +2640,7 @@ do_break_directory(GConfEngine* conf, const gchar** args)
 }
 
 static int
-do_get_default_source(GConfEngine* conf, const gchar** args)
+do_get_default_source (const gchar** args)
 {
   gchar *source;
   gchar buf[512];
